package com.persagy.dmp.rwd.edit.service;

import com.fasterxml.jackson.databind.node.ObjectNode;
import com.persagy.common.criteria.CriteriaUtils;
import com.persagy.common.criteria.JacksonCriteria;
import com.persagy.common.web.ListResponse;
import com.persagy.common.web.MapResponse;
import com.persagy.common.web.PagedResponse;
import com.persagy.dmp.rwd.edit.config.web.UserUtils;
import com.persagy.dmp.rwd.edit.entity.ClassDef;
import com.persagy.dmp.rwd.edit.entity.ClassDefChangeRecord;
import com.persagy.dmp.rwd.edit.entity.QClassDefChangeRecord;
import com.persagy.dmp.rwd.edit.enumeration.EnumOperationType;
import com.persagy.dmp.rwd.edit.enumeration.EnumVersionState;
import com.persagy.dmp.rwd.edit.repository.ClassDefChangeRecordRepository;
import com.persagy.dmp.rwd.edit.repository.ClassDefRepository;
import com.persagy.dmp.rwd.model.ClassDefModel;
import com.querydsl.core.types.dsl.BooleanExpression;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.transaction.Transactional;
import java.util.*;

@Slf4j
@Service
public class ClassDefChangeRecordService {

    @Autowired
    ClassDefChangeRecordRepository repository;

    @Autowired
    ClassDefService classDefService;

    @Autowired
    CriteriaUtils criteriaUtils;

    @Autowired
    ClassDefRepository classDefRepository;

    private List<BooleanExpression> parse(ObjectNode object) {
        List<BooleanExpression> list = new LinkedList<>();
        QClassDefChangeRecord qt = QClassDefChangeRecord.classDefChangeRecord;
        list.addAll(CriteriaUtils.parse(qt.id, object.get("id")));
        list.addAll(CriteriaUtils.parse(qt.code, object.get("code")));
        list.addAll(CriteriaUtils.parse(qt.majorCode, object.get("majorCode")));
        list.addAll(CriteriaUtils.parse(qt.systemCode, object.get("systemCode")));
        list.addAll(CriteriaUtils.parse(qt.equipmentCode, object.get("equipmentCode")));
        list.addAll(CriteriaUtils.parse(qt.parentCode, object.get("parentCode")));
        list.addAll(CriteriaUtils.parse(qt.operationType, object.get("operationType"), EnumOperationType.class));
        list.addAll(CriteriaUtils.parse(qt.operationUser, object.get("operationUser")));
        list.addAll(CriteriaUtils.parse(qt.operationTime, object.get("operationTime")));
        list.addAll(CriteriaUtils.parse(qt.confirmUser, object.get("confirmUser")));
        list.addAll(CriteriaUtils.parse(qt.confirmTime, object.get("confirmTime")));
        list.addAll(CriteriaUtils.parse(qt.publishUser, object.get("publishUser")));
        list.addAll(CriteriaUtils.parse(qt.publishTime, object.get("publishTime")));
        list.addAll(CriteriaUtils.parse(qt.finishTime, object.get("finishTime")));
        list.addAll(CriteriaUtils.parse(qt.state, object.get("state"), EnumVersionState.class));
        list.addAll(CriteriaUtils.parse(qt.version, object.get("version")));
        return list;
    }

    public PagedResponse<ClassDefChangeRecord> query(JacksonCriteria criteria) {
        return criteriaUtils.query(QClassDefChangeRecord.classDefChangeRecord, this::parse, criteria);
    }

    @Transactional
    public MapResponse create(ClassDefChangeRecord param) {
        MapResponse response = new MapResponse();
        String code = param.getCode();
        if (code == null) {
            response.setFail("code is required");
            return response;
        }
        List<ClassDefModel> classDefList = queryClassDef(code, null, null);
        if (EnumOperationType.create.equals(param.getOperationType())) {
            if (classDefList.size() > 0) {
                response.setFail("code is exists");
                return response;
            }
        } else {
            if (classDefList.size() == 0) {
                response.setFail("code not exists");
                return response;
            }
            ClassDefModel classDef = classDefList.get(0);
            param.setGroupCode(classDef.getGroupCode());
            param.setObjType(classDef.getObjType());
            param.setName(classDef.getName());
            if (param.getAliasCode() == null) {
                param.setAliasCode(classDef.getAliasCode());
            }
            if (param.getAliasName() == null) {
                param.setAliasName(classDef.getAliasName());
            }
            param.setType(classDef.getType());
            param.setProjectId(classDef.getProjectId());
            param.setMajorCode(classDef.getMajorCode());
            param.setSystemCode(classDef.getSystemCode());
            param.setEquipmentCode(classDef.getEquipmentCode());
            param.setParentCode(classDef.getParentCode());
        }
        QClassDefChangeRecord qt = QClassDefChangeRecord.classDefChangeRecord;
        BooleanExpression ex = qt.code.eq(code).and(qt.state.eq(EnumVersionState.INIT)).and(qt.version.isNull());
        Iterable<ClassDefChangeRecord> all = repository.findAll(ex);
        List<ClassDefChangeRecord> list = new ArrayList<>();
        all.forEach(list::add);
        if (list.size() > 0) {
            repository.deleteAll(list);
        }
        param.setOperationUser(UserUtils.currentUserId() + "");
        param.setOperationTime(new Date());
        param.setValid(true);
        param.setState(EnumVersionState.INIT);
        repository.save(param);
        response.add("id", param.getId());
        return response;
    }

    @Transactional
    public MapResponse update(ClassDefChangeRecord param) {
        MapResponse response = new MapResponse();
        Optional<ClassDefChangeRecord> optional = repository.findById(param.getId());
        if (!optional.isPresent()) {
            response.setFail("code not exists");
            return response;
        }
        ClassDefChangeRecord changeRecord = optional.get();
        if (EnumVersionState.INIT.equals(changeRecord.getState()) && changeRecord.getVersion() == null) {
            if (param.getAliasCode() != null) {
                changeRecord.setAliasCode(param.getAliasCode());
            }
            if (param.getAliasName() != null) {
                changeRecord.setAliasName(param.getAliasName());
            }
        }
        repository.save(changeRecord);
        return response;
    }

    @Transactional
    public MapResponse addVersion(String version, List<Integer> idList) {
        MapResponse response = new MapResponse();
        Integer num = repository.addVersion(version, idList);
        response.setMessage("update num: " + num);
        return response;
    }

    @Transactional
    public MapResponse delete(ClassDefChangeRecord param) {
        MapResponse response = new MapResponse();
        Optional<ClassDefChangeRecord> optional = repository.findById(param.getId());
        if (optional.isPresent()) {
            ClassDefChangeRecord changeRecord = optional.get();
            if (!EnumVersionState.INIT.equals(changeRecord.getState()) || changeRecord.getVersion() != null) {
                response.setFail(changeRecord.getId() + " 不可删除！");
                return response;
            }
        }
        repository.delete(param);
        return response;
    }

    public List<String> getSqlList(String version, Boolean isTotal) {
        List<String> sqlList = new ArrayList<>();
        JacksonCriteria criteria = JacksonCriteria.newInstance();
        criteria.add("version", version);
        PagedResponse<ClassDefChangeRecord> response = query(criteria);
        List<ClassDefChangeRecord> data = response.getData();
        if (isTotal) {
            List<ClassDefModel> classDefModelList = queryClassDef(null, null, version);
            if (classDefModelList != null) {
                for (ClassDefModel classDefModel : classDefModelList) {
                    ClassDefChangeRecord changeRecord = new ClassDefChangeRecord();
                    changeRecord.setCode(classDefModel.getCode());
                    changeRecord.setGroupCode(classDefModel.getGroupCode());
                    changeRecord.setObjType(classDefModel.getObjType());
                    changeRecord.setName(classDefModel.getName());
                    changeRecord.setAliasCode(classDefModel.getAliasCode());
                    changeRecord.setAliasName(classDefModel.getAliasName());
                    changeRecord.setType(classDefModel.getType());
                    changeRecord.setProjectId(classDefModel.getProjectId());
                    changeRecord.setMajorCode(classDefModel.getMajorCode());
                    changeRecord.setSystemCode(classDefModel.getSystemCode());
                    changeRecord.setEquipmentCode(classDefModel.getEquipmentCode());
                    changeRecord.setParentCode(classDefModel.getParentCode());
                    changeRecord.setOperationType(EnumOperationType.create);
                    data.add(changeRecord);
                }
            }
        }
        for (ClassDefChangeRecord changeRecord : data) {
            StringBuilder sql = new StringBuilder();
            switch (changeRecord.getOperationType()) {
                case create: {
                    String uuid = UUID.randomUUID().toString().replace("-", "");
                    sql.append("INSERT INTO `rwd_def_class` (`id`,`code`,`obj_type`,`name`,`type`,`group_code`,`project_id`,`alias_code`,`alias_name`,`major_code`,`system_code`,`equipment_code`,`parent_code`) VALUES (");
                    sql.append("'" + uuid + "',");
                    sql.append("'" + changeRecord.getCode() + "',");
                    if (changeRecord.getObjType() == null) {
                        sql.append("null,");
                    } else {
                        sql.append("'" + changeRecord.getObjType().name() + "',");
                    }
                    sql.append("'" + changeRecord.getName() + "',");
                    if (changeRecord.getType() == null) {
                        sql.append("null,");
                    } else {
                        sql.append("'" + changeRecord.getType() + "',");
                    }
                    sql.append("'" + changeRecord.getGroupCode() + "',");
                    sql.append("'" + changeRecord.getProjectId() + "',");
                    sql.append("'" + changeRecord.getAliasCode() + "',");
                    sql.append("'" + changeRecord.getAliasName() + "',");
                    if (changeRecord.getMajorCode() == null) {
                        sql.append("null,");
                    } else {
                        sql.append("'" + changeRecord.getMajorCode() + "',");
                    }
                    if (changeRecord.getSystemCode() == null) {
                        sql.append("null,");
                    } else {
                        sql.append("'" + changeRecord.getSystemCode() + "',");
                    }
                    if (changeRecord.getEquipmentCode() == null) {
                        sql.append("null,");
                    } else {
                        sql.append("'" + changeRecord.getEquipmentCode() + "',");
                    }
                    if (changeRecord.getParentCode() == null) {
                        sql.append("null);\n");
                    } else {
                        sql.append("'" + changeRecord.getParentCode() + "');\n");
                    }
                    break;
                }
                case update: {
                    sql.append("UPDATE `rwd_def_class` SET");
                    sql.append(" `alias_code` = '" + changeRecord.getAliasCode() + "',");
                    sql.append(" `alias_name` = '" + changeRecord.getAliasName() + "'");
                    sql.append(" WHERE `code` = '" + changeRecord.getCode() + "'");
                    sql.append(" and `group_code` = '" + changeRecord.getGroupCode() + "'");
                    sql.append(" and `project_id` = '" + changeRecord.getProjectId() + "';\n");
                    break;
                }
                case delete: {
                    sql.append("DELETE FROM `rwd_def_class`");
                    sql.append(" WHERE `code` = '" + changeRecord.getCode() + "'");
                    sql.append(" and `group_code` = '" + changeRecord.getGroupCode() + "'");
                    sql.append(" and `project_id` = '" + changeRecord.getProjectId() + "';\n");
                    break;
                }
                default: {
                    break;
                }
            }
            sqlList.add(sql.toString());
        }
        return sqlList;
    }

    public void updateStateByVersion(EnumVersionState state, String version) {
        JacksonCriteria criteria = JacksonCriteria.newInstance();
        criteria.add("version", version);
        PagedResponse<ClassDefChangeRecord> query = query(criteria);
        if (query.getCount() == 0) {
            return;
        }
        List<ClassDefChangeRecord> data = query.getData();
        for (ClassDefChangeRecord changeRecord : data) {
            switch (state) {
                case CONFIRM: {
                    changeRecord.setConfirmUser(UserUtils.currentUserId() + "");
                    changeRecord.setConfirmTime(new Date());
                    changeRecord.setState(state);
                    break;
                }
                case PUBLISH: {
                    changeRecord.setPublishUser(UserUtils.currentUserId() + "");
                    changeRecord.setPublishTime(new Date());
                    changeRecord.setState(state);
                    break;
                }
                case FINISH: {
                    changeRecord.setFinishTime(new Date());
                    changeRecord.setState(state);
                    break;
                }
                default: {
                    changeRecord.setState(state);
                    break;
                }
            }
        }
        if (state == EnumVersionState.FINISH) {
            updateClassDef(data, version);
        }
        repository.saveAll(data);
    }

    private void updateClassDef(List<ClassDefChangeRecord> data, String version) {
        List<ClassDef> classDefList = new ArrayList<>();
        List<String> codeList = new ArrayList<>();
        data.forEach(item -> {
            String uuid = UUID.randomUUID().toString().replace("-", "");
            ClassDef classDef = new ClassDef();
            classDef.setId(uuid);
            classDef.setCode(item.getCode());
            classDef.setObjType(item.getObjType());
            classDef.setName(item.getName());
            classDef.setType(item.getType());
            classDef.setGroupCode(item.getGroupCode());
            classDef.setProjectId(item.getProjectId());
            classDef.setAliasCode(item.getAliasCode());
            classDef.setAliasName(item.getAliasName());
            classDef.setMajorCode(item.getMajorCode());
            classDef.setSystemCode(item.getSystemCode());
            classDef.setEquipmentCode(item.getEquipmentCode());
            classDef.setParentCode(item.getParentCode());
            classDef.setIsUsed(1);
            classDef.setVersion(version);
            classDef.setLastUpdateTime(new Date());
            classDefList.add(classDef);
            codeList.add(item.getCode());
        });
        List<ClassDefModel> classDefModelList = queryClassDef(null, codeList, null);
        if (classDefModelList != null) {
            classDefModelList.forEach(item -> {
                item.setIsUsed(0);
                item.setLastUpdateTime(new Date());
                classDefList.add(ClassDef.fromModel(item));
            });
        }
        classDefRepository.saveAll(classDefList);
    }

    private List<ClassDefModel> queryClassDef(String code, List<String> codeList, String neVersion) {
        JacksonCriteria criteria = JacksonCriteria.newInstance();
        criteria.add("type", "common");
        if (code != null) {
            criteria.add("code", code);
        }
        if (codeList != null) {
            criteria.add("code").in(codeList);
        }
        if (neVersion != null) {
            criteria.add("version").ne(neVersion);
        }
        ListResponse<ClassDefModel> query = classDefService.queryClass(criteria);
        List<ClassDefModel> data = query.getData();
        return data;
    }
}

