package com.persagy.dmp.rwd.edit;

import com.persagy.common.criteria.JacksonCriteria;
import com.persagy.common.json.JacksonMapper;
import com.persagy.common.web.ListResponse;
import com.persagy.common.web.MapResponse;
import com.persagy.common.web.PagedResponse;
import com.persagy.dmp.rwd.edit.entity.ClassDefChangeRecord;
import com.persagy.dmp.rwd.edit.enumeration.EnumOperationType;
import com.persagy.dmp.rwd.edit.service.ClassDefChangeRecordService;
import com.persagy.dmp.rwd.edit.service.ClassDefService;
import com.persagy.dmp.rwd.enums.ObjType;
import com.persagy.dmp.rwd.model.ClassDefModel;
import lombok.extern.slf4j.Slf4j;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.test.context.junit4.SpringRunner;

import java.util.ArrayList;
import java.util.List;

@Slf4j
@SpringBootTest
@RunWith(SpringRunner.class)
public class ClassDefTest {

    @Autowired
    private ClassDefService classDefService;

    @Autowired
    private ClassDefChangeRecordService classDefChangeRecordService;

    /**
     * 查询类型数据
     */
    @Test
    public void queryClass() {
        JacksonCriteria criteria = JacksonCriteria.newInstance();
        criteria.add("parentCode").isNull(true);
        criteria.add("type", "common");
        ListResponse<ClassDefModel> response = classDefService.queryClass(criteria);
        System.err.println(JacksonMapper.toFormatJson(response));
    }

    /**
     * 查询类型变更记录
     */
    @Test
    public void query() {
        JacksonCriteria criteria = JacksonCriteria.newInstance();
        criteria.add("type", "common");
        PagedResponse<ClassDefChangeRecord> query = classDefChangeRecordService.query(criteria);
        System.err.println(JacksonMapper.toFormatJson(query));
    }

    /**
     * 创建类型变更记录
     */
    @Test
    public void create() {
        ClassDefChangeRecord param = new ClassDefChangeRecord();
        param.setCode("eq01");
        param.setName("eq01");
        param.setGroupCode("0");
        param.setObjType(ObjType.component);
        param.setAliasCode("eq01");
        param.setAliasName("eq01");
        param.setType("common");
        param.setProjectId("0");
        param.setMajorCode("AC");
        param.setSystemCode("ACVT");
        param.setEquipmentCode("ACVTSF");
        param.setParentCode("component");
        param.setOperationType(EnumOperationType.create);
        MapResponse response = classDefChangeRecordService.create(param);
        System.err.println(JacksonMapper.toFormatJson(response));
    }

    /**
     * 修改类型变更记录
     */
    @Test
    public void update() {
        ClassDefChangeRecord param = new ClassDefChangeRecord();
        param.setId(2);
        param.setCode("eq01");
        param.setAliasCode("eq01");
        param.setAliasName("eq02");
        MapResponse response = classDefChangeRecordService.update(param);
        System.err.println(JacksonMapper.toFormatJson(response));
    }

    /**
     * 类型变更记录添加版本
     */
    @Test
    public void addVersion() {
        String version = "1.0.0";
        List<Integer> idList = new ArrayList<>();
        idList.add(2);
        MapResponse response = classDefChangeRecordService.addVersion(version, idList);
        System.err.println(JacksonMapper.toFormatJson(response));
    }

    /**
     * 删除类型变更记录
     */
    @Test
    public void delete() {
        ClassDefChangeRecord param = new ClassDefChangeRecord();
        param.setId(2);
        MapResponse response = classDefChangeRecordService.delete(param);
        System.err.println(JacksonMapper.toFormatJson(response));
    }

}
